package jogl.example;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.util.Scanner;

import javax.swing.Timer;

import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.awt.GLCanvas;

class Renderer implements GLEventListener, KeyListener, ActionListener {
	// Atributos da classe
	private GLCanvas display;
	private Timer animationTimer;
	private boolean animating;

	// Variaveis do usuario
	Poligono mapa, convexHull, conjuntoDePonto;
	Ponto min, max;
	
	
	public Renderer(GLCanvas display) {
		this.display = display;
		
		startAnimation();
		
		// Inicia variaveis 
		mapa = new Poligono();
		convexHull = new Poligono();
		conjuntoDePonto = new Poligono();
		min = new Ponto(0, 0);
		max = new Ponto(10, 10);
	}
	
	// ###################  Funcoes do usuario  ###################
	
	// **********************************************************************
	//  Calcula o produto escalar entre os vetores V1 e V2
	//**********************************************************************

	private double prodEscalar(Ponto v1, Ponto v2) {
		return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
	}
	
	// **********************************************************************
	//  Calcula o produto vetorial entre os vetores V1 e V2
	//**********************************************************************
	private Ponto prodVetorial(Ponto v1, Ponto v2) {
		Ponto vresult = new Ponto();
	    vresult.x = v1.y * v2.z - (v1.z * v2.y);
	    vresult.y = v1.z * v2.x - (v1.x * v2.z);
	    vresult.z = v1.x * v2.y - (v1.y * v2.x);
	    return vresult;
	}
	

	
	/* ********************************************************************** */
	/*                                                                        */
	/*  Calcula a interseccao entre 2 retas (no plano "XY" Z = 0)             */
	/*                                                                        */
	/* k : ponto inicial da reta 1                                            */
	/* l : ponto final da reta 1                                              */
	/* m : ponto inicial da reta 2                                            */
	/* n : ponto final da reta 2                                              */
	/*                                                                        */
	/* s: valor do par‰metro no ponto de interse‹o (sobre a reta KL)         */
	/* t: valor do par‰metro no ponto de interse‹o (sobre a reta MN)         */
	/*                                                                        */
	/* ********************************************************************** */
	public ResultadoInterseccao intersec2d(Ponto k, Ponto l, Ponto m, Ponto n)
	{
	    double det, s, t;
	    
	    det = (n.x - m.x) * (l.y - k.y)  -  (n.y - m.y) * (l.x - k.x);
	    
	    if (det == 0.0)
	        return new ResultadoInterseccao(false, 0, 0);
	    
	    s = ((n.x - m.x) * (m.y - k.y) - (n.y - m.y) * (m.x - k.x))/ det ;
	    t = ((l.x - k.x) * (m.y - k.y) - (l.y - k.y) * (m.x - k.x))/ det ;
	    
	    return new ResultadoInterseccao(true, s, t);
	}
	
	public boolean HaInterseccao(Ponto k, Ponto l, Ponto m, Ponto n)
	{
	    ResultadoInterseccao resultado;

	    resultado = intersec2d( k,  l,  m,  n);
	    if (!resultado.resultado) return false;
	    if (resultado.s>=0.0 && resultado.s <=1.0 && resultado.t>=0.0 && resultado.t<=1.0)
	        return true;
	    else return false;
	}
	
	private void leMapa(String nome) {
		try {
			Scanner input = new Scanner(new File(nome));
			
			if (!input.hasNext()) {
				System.out.println("Erro ao abrir " + nome);
				return;
			}
			
			input.useDelimiter("\\s+");
			System.out.println("Lendo arquivo " + nome + "...");
			
			String s;
			int nLinha = 0;
			int qtdVertices;
			double x, y;
			
			qtdVertices = input.nextInt();
			x = Double.parseDouble(input.next());
			y = Double.parseDouble(input.next());
			
			min = new Ponto(x, y);
			max = new Ponto(x, y);
			mapa.insereVertice(new Ponto(x, y));
			
			for (int i=0; i<qtdVertices-1; i++) {
				x = Double.parseDouble(input.next());
				y = Double.parseDouble(input.next());
				
				if (x<min.x) min.x = x;
				if (y<min.y) min.y = y;
				
				if (x>max.x) max.x = x;
				if (y>max.y) max.y = y;
				
				nLinha++;
				mapa.insereVertice(new Ponto(x, y));
			}
			
			input.close();
			
			System.out.println("Leitura concluida.");
			System.out.println("Minimo: " + min + "Maximo: " + max);
		}catch (Exception e) {
			System.out.println("Erro desconhecido: " + e.getMessage());
			e.printStackTrace();
		}
	}
	

	// **********************************************************************
	//
	// **********************************************************************
	private void desenhaEixos(GLAutoDrawable gLDrawable) {
		final GL2 gl = gLDrawable.getGL().getGL2();
		
	    Ponto meio = new Ponto();
	    meio.x = (max.x+min.x)/2;
	    meio.y = (max.y+min.y)/2;
	    meio.z = (max.z+min.z)/2;

	    gl.glBegin(gl.GL_LINES);
	    	//  eixo horizontal
	        gl.glVertex2d(min.x,meio.y);
	        gl.glVertex2d(max.x,meio.y);
	        //  eixo vertical
	        gl.glVertex2d(meio.x,min.y);
	        gl.glVertex2d(meio.x,max.y);
	    gl.glEnd();
	}
	
	// #################################################################
	
	public void display(GLAutoDrawable gLDrawable) {
		final GL2 gl = gLDrawable.getGL().getGL2();

		// Limpa a tela coma cor de fundo
		gl.glClear(GL2.GL_COLOR_BUFFER_BIT);
		
		// Define os limites lógicos da área OpenGL dentro da Janela
		gl.glMatrixMode(GL2.GL_MODELVIEW0_ARB);
		gl.glLoadIdentity();

		// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		// Coloque aqui as chamadas das rotinas que desenha os objetos
		// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

		gl.glLineWidth(1);
		gl.glColor3f(1, 1, 1);
		desenhaEixos(gLDrawable);

		gl.glLineWidth(2);
		gl.glColor3f(1, 1, 0);
		mapa.desenhaPoligono(gLDrawable);

		gl.glFlush();
	}

	public void init(GLAutoDrawable gLDrawable) {
		System.out.println("init() called");

		GL2 gl = gLDrawable.getGL().getGL2();
		gl.glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
		
		leMapa("EstadoRS.txt");
		
		FPSCounter.StartCounter();
	}

	// **********************************************************************
	//  void init(void)
	//  Inicializa os parametros globais de OpenGL
	// **********************************************************************
	public void initOLD(GLAutoDrawable gLDrawable)
	{
		GL2 gl = gLDrawable.getGL().getGL2();
		// Define a cor do fundo da tela (AZUL)
		gl.glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
	
	    // Leitura do arquivo
	    mapa.insereVertice(new Ponto(0,0));
	    mapa.insereVertice(new Ponto(-3,5));
	    mapa.insereVertice(new Ponto(5,5));
	    mapa.insereVertice(new Ponto(8,-3));
	    mapa.insereVertice(new Ponto(-4,-3));
	
	    // Seta os limites da ‡rea de desenho
	    min = new Ponto(-10, -10, 0);
	    max = new Ponto( 10,  10, 1);
	}
	
	public void reshape(GLAutoDrawable gLDrawable, int x, int y, int width, int height) {
		System.out.println("reshape() called: x = " + x + ", y = " + y + ", width = " + width + ", height = " + height);
		final GL2 gl = gLDrawable.getGL().getGL2();

		// Reset the coordinate system before modifying
		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		// Define a area a ser ocupada pela área OpenGL dentro da Janela
		gl.glViewport(0, 0, width, height);
		gl.glOrtho(min.x, max.x, min.y, max.y, 0, 1);
		
		// Define os limites lógicos da área OpenGL dentro da Janela
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	public void dispose(GLAutoDrawable arg0) {
		System.out.println("dispose() called");
	}

	@Override
	public void keyPressed(KeyEvent e) {
		int key = e.getKeyCode();

		switch (key) {
		case KeyEvent.VK_ESCAPE:
			System.exit(0);
		
		case KeyEvent.VK_UP:
			// TODO
			break;
		case KeyEvent.VK_DOWN:
			// TODO
			break;

		default:
			break;
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub

	}

	private void Animate() {
		FPSCounter.StopAndPost();
		for (int i = 0; i < 1000; i++) {
			HaInterseccao(new Ponto(i,10), new Ponto(5,5*i), new Ponto(3,3), new Ponto(4,67));
		}
	}

	public void startAnimation() {
		if (!animating) {
			if (animationTimer == null) {
				animationTimer = new Timer(0, this);
			}
			animationTimer.start();
			animating = true;
		}
	}

	public void pauseAnimation() {
		if (animating) {
			animationTimer.stop();
			animating = false;
		}
	}

	/** Funcao chamada a cada frame **/
	@Override
	public void actionPerformed(ActionEvent e) {
		Animate();
        display.repaint();
	}
}